from rich.console import Console
from rich.live import Live
from rich.panel import Panel
from rich.table import Table
from rich.text import Text
from time import sleep


class Output:
    """
    Classe Output : Gestion de l'affichage pour le jeu de quiz.
    """

    def __init__(self) -> None:
        self.logo_name = "Quiz Game"
        self.console = Console()

    def info(self, player: str, score: int, life: int):
        """Affiche les informations générales sur le joueur."""
        table = Table(show_header=False, box=None)
        table.add_row("Gamer:", f"[bold green]{player}[/bold green]")
        table.add_row("Score:", f"[bold yellow]{score}[/bold yellow]")
        table.add_row("Vie:", f"[red]{':heart:' * life}[/red]")

        panel = Panel(table, border_style="bold blue")
        self.console.print(panel, justify="center")

    def show_quiz(self, question, reponses):
        """Affiche la question et les réponses du quiz."""
        self.console.print(Panel(f"{question} ?", border_style="bold blue"), justify="center")

        table = Table(show_header=False, box=None)
        line = []

        for i, reponse in enumerate(reponses, 1):
            line.append(Panel(f"{i} - {reponse}", border_style="bold blue"))
            if i % 2 == 0 or i == len(reponses):
                table.add_row(*line)
                line = []

        self.console.print(table, justify="center")

    def prompt(self, start: int = 1, end: int = 4) -> int:
        """Demande la réponse du joueur."""
        while True:
            value = self.console.input(f"Quelle est la [bold green]bonne réponse[/bold green] (ou entrez 'end' pour quitter) : ")

            if value.lower() in ["end", "."]:
                return 0

            if not value.isdigit():
                self.console.print(f"[bold red]Entrez un nombre entre {start} et {end} ![/bold red]")
                continue

            value = int(value)
            if start <= value <= end:
                return value

            self.console.print(f"[bold red]Entrez une valeur valide entre {start} et {end} ![/bold red]")

    def read_event_touch(self):
        """Pause jusqu'à ce que l'utilisateur appuie sur Entrée."""
        self.console.input("\n[bold green]Appuyez sur Entrée pour continuer...[/bold green]")

    def good_reponse_animation(self, text: str = "Bonne réponse !"):
        """Animation pour une bonne réponse."""
        self.console.clear()

        message = Text(text, justify="center", style="bold green")
        with Live(message, refresh_per_second=4, console=self.console) as live:
            for i in range(5):
                if i % 2 == 0:
                    message.stylize("on green")
                else:
                    message.stylize("on yellow")
                live.update(message)
                sleep(0.3)

    def bad_reponse_animation(self, reponse):
        """Animation pour une mauvaise réponse."""
        self.console.clear()
        self.console.print("Mauvaise réponse !", justify="center", style="bold red on red")
        self.console.print(
            Panel(f"La bonne réponse était [bold green]{reponse}[/bold green]", border_style="cyan"),
            justify="center"
        )
        self.read_event_touch()

    def show_stat(self, data: dict):
        """Affiche les statistiques finales du joueur."""
        self.logo()
        print("\n")

        panel = Panel(f"[bold black]{data['name']}[/bold black]", style="on white", border_style="")
        self.console.print(panel, justify="center")
        self.console.print("Mes performances lors de cette partie :", justify="center")

        table = Table(show_header=False, box=None)

        # Ajout des données au tableau
        self.table_add_row(table, "Questions répondues", data['answered'])
        self.table_add_row(table, "Questions trouvées", data['correct'], highlight=data['correct'] < data['incorrect'])
        self.table_add_row(table, "Questions perdues", data['incorrect'], highlight=data['correct'] > data['incorrect'])
        self.table_add_row(table, "Score", data['score'])
        self.table_add_row(table, "Bonnes réponses successives", data['max_successive_correct'])
        self.table_add_row(table, "Vies restantes", data['remaining_life'])

        self.console.print(Panel(table, border_style="bold blue"), justify="center")

    def table_add_row(self, table: Table, key: str, value: int, highlight: bool = False):
        """Ajoute une ligne au tableau avec style conditionnel."""
        style = "red" if value == 0 or highlight else "green"
        table.add_row(
            Panel(key, border_style="bold blue"),
            Panel(str(value), style=f"bold {style}", border_style=f"bold {style}")
        )

    def logo(self):
        """Affiche le logo du jeu."""
        panel = Panel(self.logo_name, style="green bold", border_style="bold blue")
        self.console.print(panel, justify="center")



from output import Output
from random import shuffle
from json import dump, load
from pathlib import Path



output = Output()


class Quiz:
    def __init__(self, questions: dict, player: str, life: int = 3, score: int = 0, path_save: str = "data.json") -> None:
        """
        # Quiz
        est une application console interactive, 
        conçue pour faciler la mise en place de **Game Quiz** amusant.

        Parameters
        ----------
        questions : dict
            Dictionnaire des questions et réponses.
        player : str
            Nom du joueur.
        life : int, optional
            Nombre initial de vies (par défaut : 3).
        score : int, optional
            Score initial (par défaut : 0).
        path_save : str, optional
            Chemin pour sauvegarder les statistiques (par défaut : 'data.json').
        """
        
        if not isinstance(questions, dict):
            raise ValueError("Le paramètre 'questions' doit être un dictionnaire.")

        self.output = output
        self.questions = questions
        self.player = player
        self.life = life if life > 0 else 3
        self.init_life = self.life
        self.score = score

        self.count_success = 0
        self.count_responses = 0
        self.count_correct_responses = 0
        self.max_successive_correct = 0

        self.path_save = Path(path_save)
        
    def shuffle_questions(self):
        """Mélange les questions et réponses."""
        data = list(self.questions.items())
        shuffle(data)
        self.questions = dict(data)

    def play(self):
        """Lancer le quiz."""
        self.shuffle_questions()

        for question, answers in self.questions.items():
            self.output.console.clear()

            # Mise à jour des statistiques
            self.increment_count_responses()

            # Affichage des informations
            self.output.info(self.player, self.score, self.life)

            # Préparation de la question et des réponses
            correct_answer = answers[0]
            shuffle(answers)

            # Affichage de la question
            self.output.show_quiz(question, answers)

            # Saisie du joueur
            player_response = self.output.prompt(end=len(answers))

            # Arrêt du jeu si le joueur choisit 0
            if player_response == 0:
                break

            # Vérification de la réponse
            if answers[player_response - 1] == correct_answer:
                self.handle_correct_answer()
            else:
                self.handle_wrong_answer(correct_answer)

            # Fin du jeu si les vies sont épuisées
            if self.life == 0:
                break

        # Affichage des statistiques finales
        self.end_game()

    def handle_correct_answer(self):
        """Gérer une réponse correcte."""
        self.output.good_reponse_animation()
        self.increment_score()
        self.increment_count_success()
        self.increment_count_correct_responses()
        self.check_bonus_life()

    def handle_wrong_answer(self, correct_answer):
        """Gérer une réponse incorrecte."""
        self.output.bad_reponse_animation(correct_answer)
        self.decrement_score()
        self.decrement_life()
        self.update_max_successive_correct()
        self.reset_count_success()

    def check_bonus_life(self):
        """Accorder un bonus de vie toutes les 5 réponses correctes consécutives."""
        if self.count_success % 5 == 0 and self.life < self.init_life:
            self.increment_life()
            self.output.good_reponse_animation(text="Vous gagnez une vie supplémentaire ! ❤️")

    def end_game(self):
        """Afficher les statistiques et sauvegarder les données."""
        self.output.console.clear()
        player_data = self.save()
        self.output.show_stat(player_data)

    def increment_score(self):
        self.score += 1

    def decrement_score(self):
        if self.score > 0:
            self.score -= 1

    def increment_life(self):
        self.life += 1

    def decrement_life(self):
        self.life -= 1

    def increment_count_responses(self):
        self.count_responses += 1

    def increment_count_correct_responses(self):
        self.count_correct_responses += 1

    def increment_count_success(self):
        self.count_success += 1

    def reset_count_success(self):
        self.count_success = 0

    def update_max_successive_correct(self):
        if self.count_success > self.max_successive_correct:
            self.max_successive_correct = self.count_success

    def save(self):
        """Sauvegarder les données du joueur dans un fichier JSON."""
        try:
            with open(self.path_save, 'r', encoding='utf-8') as file:
                data = load(file)
        except (FileNotFoundError, ValueError):
            data = []

        # Ajout des données du joueur
        player_data = self.get_player_data()
        data.append(player_data)

        # Sauvegarde dans le fichier
        with open(self.path_save, 'w', encoding='utf-8') as file:
            dump(data, file, indent=4)

        return player_data

    def get_player_data(self):
        """Créer un dictionnaire contenant les données du joueur."""
        return {
            'name': self.player,
            'answered': self.count_responses,
            'correct': self.count_correct_responses,
            'incorrect': self.count_responses - self.count_correct_responses,
            'score': self.score,
            'max_successive_correct': self.max_successive_correct,
            'remaining_life': self.life,
        }

def main():
    # Code pour démarrer le quiz
    from quizen import Quiz  # importez la classe Quiz
    from json import load

    path = 'data/questions.json'

    # Charger les questions depuis un fichier JSON
    with open(path, 'r', encoding='utf-8') as file:
        questions = load(file)

    # Lancer le quiz avec le nom du joueur
    Quiz(questions, player="Joueur").play()

if __name__ == '__main__':
    main()


from setuptools import setup, find_packages

setup(
    name='quizen',                          # Nom du paquet
    version='0.1.0',                       # Version du paquet
    packages=find_packages(),              # Inclut tous les sous-dossiers contenant __init__.py
    install_requires=[                     # Liste des dépendances du paquet
        'rich',                             # Affichage dans la console (exemple)
    ],
    entry_points={                         # Point d'entrée si nécessaire (pour les commandes CLI)
        'console_scripts': [
            'quizen = quizen.quiz:main',         # Point d'entrée principal pour le jeu de quiz
        ],
    },
    author='kouya tosten',
    author_email='kouyatosten@gmail.com', # Votre email
    description='Un module pour créer facilement un jeu de quiz',  # Brève description
    long_description=open('README.md', encoding='utf-8').read(),  # Lire le README.md pour la description longue
    long_description_content_type='text/markdown',  # Format du fichier README
    url='https://github.com/Tostenn/Quiz.git',  # Lien vers le dépôt GitHub
    classifiers=[                          # Classifiers PyPI pour mieux catégoriser le paquet
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.6',                # Version minimale de Python requise
)
